#!/usr/bin/env python3

from typing import Dict, Optional

from .transport import Transport


class HTTPTransport(Transport):
    def __init__(self, url: str, headers: Optional[Dict[str, str]] = None) -> None:
        self.url: str = url
        self.headers = headers


class HTTPException(Exception):
    def __init__(self, status_code: int, text: str) -> None:
        message = (
            f"Http request failed with status_code={status_code} and message: {text}"
        )
        super(HTTPException, self).__init__(message)
        self.status_code = status_code
        self.text = text



#!/usr/bin/env python3

from abc import ABC, abstractmethod
from typing import Any, Dict

from unicodecsv import writer
from unicodecsv.py3 import UnicodeWriter


class FailedOperationException(Exception):
    def __init__(
        self,
        reporter: "Reporter",
        err_msg: str,
        operation_name: str,
        variables: Dict[str, Any],
    ) -> None:
        self.reporter = reporter
        self.err_msg = err_msg
        self.operation_name = operation_name
        self.variables = variables

    def log_failed_operation(self, row_identifier: str, row: Dict[str, Any]) -> None:
        self.reporter.log_failed_operation(row_identifier, row, self)

    def __str__(self) -> str:
        return self.err_msg


class Reporter(ABC):
    @abstractmethod
    def log_successful_operation(
        self,
        operation_name: str,
        variables: Dict[str, Any],
        network_time: float,
        decode_time: float,
    ) -> None:
        pass

    @abstractmethod
    def log_failed_operation(
        self, row_identifier: str, row: Dict[str, Any], e: FailedOperationException
    ) -> None:
        pass


class CSVReporter(Reporter):
    def __init__(self, out_file_path: str, err_file_path: str) -> None:

        """Reporting utility for the SymphonyClient to report on
        successful and failed operations.
        In order to report on failed operation, user is required to catch
        FailedOperationException and call logFailedOperation with date
        identifier (row number) & full data for easier debugging later.

        Args:
            out_file_path (str): Path to write csv of successful operations.
            err_file_path (str): Path to write csv of failed operations.

        Example:
        ```
        reporter = CSVReporter(csvOutPath, csvErrPath)
        client = GraphqlClient("http://graph/query", session, "client/1.0", reporter=reporter)
        try:
            location = client.call(..)
        except FailedOperationException as e:
            e.logFailedOperation(data_identifier, data)
        ```

        """

        # pyre-fixme[8]: Attribute has type `UnicodeWriter`; used as `_writer`.
        self.out_file: UnicodeWriter = writer(
            # pyre-fixme[6]: Expected `_Writer` for 1st param but got `BufferedWriter`.
            open(out_file_path, "wb"),
            encoding="utf-8",
        )
        self.out_file.writerow(
            ["operation_name", "variables", "network_time", "decode_time"]
        )

        # pyre-fixme[8]: Attribute has type `UnicodeWriter`; used as `_writer`.
        self.err_file: UnicodeWriter = writer(
            # pyre-fixme[6]: Expected `_Writer` for 1st param but got `BufferedWriter`.
            open(err_file_path, "wb"),
            encoding="utf-8",
        )
        self.err_file.writerow(
            [
                "operation_name",
                "variables",
                "error_msg",
                "row_identifier",
                "row",
            ]
        )

    def log_successful_operation(
        self,
        operation_name: str,
        variables: Dict[str, Any],
        network_time: float,
        decode_time: float,
    ) -> None:
        self.out_file.writerow(
            [operation_name, str(variables), network_time, decode_time]
        )

    def log_failed_operation(
        self, row_identifier: str, row: Dict[str, Any], e: FailedOperationException
    ) -> None:
        self.err_file.writerow(
            [
                e.operation_name,
                str(e.variables),
                e.err_msg,
                row_identifier,
                str(row),
            ]
        )


class PrintReporter(Reporter):
    def log_successful_operation(
        self,
        operation_name: str,
        variables: Dict[str, Any],
        network_time: float,
        decode_time: float,
    ) -> None:
        print(
            {
                "operation_name": operation_name,
                "variables": variables,
                "network_time": network_time,
                "decode_time": decode_time,
            }
        )

    def log_failed_operation(
        self, row_identifier: str, row: Dict[str, Any], e: FailedOperationException
    ) -> None:
        print(
            {
                "operation_name": e.operation_name,
                "variables": str(e.variables),
                "error_msg": e.err_msg,
                "row_identifier": row_identifier,
                "row": str(row),
            }
        )


class DummyReporter(Reporter):
    def __init__(self) -> None:
        pass

    def log_successful_operation(
        self,
        operation_name: str,
        variables: Dict[str, Any],
        network_time: float,
        decode_time: float,
    ) -> None:
        pass

    def log_failed_operation(
        self, row_identifier: str, row: Dict[str, Any], e: FailedOperationException
    ) -> None:
        pass


DUMMY_REPORTER = DummyReporter()



#!/usr/bin/env python3

import abc
from typing import Any, Dict, Optional

from graphql.language.ast import DocumentNode


class ExtendedExecutionResult:
    def __init__(
        self,
        response: str,
        errors: Optional[Dict[str, Any]] = None,
        data: Optional[Dict[str, Any]] = None,
        extensions: Optional[Dict[str, Any]] = None,
    ) -> None:
        self.response: str = response
        self.errors: Dict[str, Any] = errors if errors is not None else {}
        self.data: Dict[str, Any] = data if data is not None else {}
        self.extensions: Dict[str, Any] = extensions if extensions is not None else {}


class Transport(abc.ABC):
    @abc.abstractmethod
    def execute(
        self, document: DocumentNode, variable_values: Dict[str, Any] = {}  # noqa: B006
    ) -> ExtendedExecutionResult:
        pass



#!/usr/bin/env python3

from dataclasses import field
from enum import Enum
from functools import partial
from typing import Type


def enum_field(enum_type: Type[Enum]):
    def encode_enum(value: Enum) -> str:
        return value.value

    def decode_enum(t: Type[Enum], value: str) -> Enum:
        return t(value)

    return field(
        metadata={
            "dataclasses_json": {
                "encoder": encode_enum,
                "decoder": partial(decode_enum, enum_type),
            }
        }
    )



#!/usr/bin/env python3

from graphql.language.ast import DocumentNode
from graphql.language.parser import parse
from graphql.language.source import Source


def gql(request_string: str) -> DocumentNode:
    if isinstance(request_string, str):
        source = Source(request_string, "GraphQL request")
        return parse(source)
    else:
        raise Exception('Received incompatible request "{}".'.format(request_string))



#!/usr/bin/env python3

from __future__ import absolute_import

import json
from datetime import datetime, timedelta, tzinfo
from enum import Enum
from typing import Any, Dict, Optional, Union

from graphql.language.ast import DocumentNode
from graphql.language.printer import print_ast
from requests.auth import AuthBase
from requests.sessions import Session

from .http import HTTPTransport, HTTPException
from .transport import ExtendedExecutionResult


class MissingEnumException(Exception):
    def __init__(self, variable: Enum) -> None:
        self.enum_type = str(type(variable))

    def __str__(self) -> str:
        return f"Try to encode missing value of enum {self.enum_type}"


class simple_utc(tzinfo):
    def tzname(self, dt: Optional[datetime]) -> Optional[str]:
        return "UTC"

    def utcoffset(self, dt: Optional[datetime]) -> Optional[timedelta]:
        return timedelta(0)


def encode_variable(
    variable: Union[datetime, Enum, object]
) -> Union[str, Dict[str, Any]]:
    if isinstance(variable, datetime):
        return datetime.isoformat(variable.replace(tzinfo=simple_utc()))
    elif isinstance(variable, Enum):
        if variable.value == "":
            raise MissingEnumException(variable)
        return variable.value
    else:
        return variable.__dict__


class RequestsHTTPSessionTransport(HTTPTransport):
    def __init__(
        self,
        session: Session,
        url: str,
        headers: Optional[Dict[str, str]] = None,
        auth: Optional[AuthBase] = None,
    ) -> None:
        """
        :param session: The session
        """
        super(RequestsHTTPSessionTransport, self).__init__(url, headers)
        self.session: Session = session
        self.auth = auth

    def execute(
        self, document: DocumentNode, variable_values: Dict[str, Any] = {}  # noqa: B006
    ) -> ExtendedExecutionResult:
        query_str = print_ast(document)
        payload = {"query": query_str, "variables": variable_values}

        response = self.session.post(
            self.url,
            data=json.dumps(payload, default=encode_variable).encode("utf-8"),
            headers=self.headers,
            auth=self.auth,
        )
        if not response.ok:
            raise HTTPException(response.status_code, response.text)

        result = response.json()
        assert (
            "errors" in result or "data" in result
        ), 'Received non-compatible response "{}"'.format(result)
        return ExtendedExecutionResult(
            response=response.text,
            errors=result.get("errors"),
            data=result.get("data"),
        )



#!/usr/bin/env python3

import warnings

from .client import Client, GraphqlDeprecationWarning, OperationException
from .gql import gql


__all__ = ["gql", "Client", "OperationException"]

warnings.filterwarnings("always", "", GraphqlDeprecationWarning)



#!/usr/bin/env python3
from typing import Dict, List

from graphql import GraphQLSchema

from .query_parser import (
    ParsedField,
    ParsedObject,
    ParsedOperation,
    ParsedQuery,
    ParsedVariableDefinition,
)
from .utils_codegen import (
    CodeChunk,
    camel_case_to_lower_case,
)


class DataclassesRenderer:
    def __init__(self, schema: GraphQLSchema) -> None:
        self.schema = schema

    def render(
        self, parsed_query: ParsedQuery, fragment_name_to_importpath: Dict[str, str]
    ) -> str:
        buffer = CodeChunk()
        buffer.write("#!/usr/bin/env python3")
        buffer.write("# @" + "generated AUTOGENERATED file. Do not Change!")
        buffer.write("")
        buffer.write("from dataclasses import dataclass")
        buffer.write("from datetime import datetime")
        buffer.write("from gql_client.runtime.datetime_utils import DATETIME_FIELD")
        buffer.write("from gql_client.runtime.graphql_client import GraphqlClient")
        buffer.write("from gql_client.runtime.client import OperationException")
        buffer.write("from gql_client.runtime.reporter import FailedOperationException")
        buffer.write("from functools import partial")
        buffer.write("from numbers import Number")
        buffer.write("from typing import Any, Callable, List, Mapping, Optional, Dict")
        buffer.write("from time import perf_counter")
        buffer.write("from dataclasses_json import DataClassJsonMixin")
        buffer.write("")
        for fragment_name in sorted(set(parsed_query.used_fragments)):
            importpath = fragment_name_to_importpath[fragment_name]
            buffer.write(
                f"from {importpath} import {fragment_name}, QUERY as {fragment_name}Query"
            )
        enum_names = set()
        for enum in parsed_query.enums:
            enum_names.add(enum.name)
        if enum_names:
            buffer.write("from gql_client.runtime.enum_utils import enum_field")
            for enum_name in sorted(enum_names):
                buffer.write(
                    f"from ..enum.{camel_case_to_lower_case(enum_name)} import {enum_name}"
                )
            buffer.write("")
        input_object_names = set()
        for input_object in parsed_query.input_objects:
            input_object_names.add(input_object.name)
        if input_object_names:
            for input_object_name in sorted(input_object_names):
                buffer.write(
                    f"from ..input.{camel_case_to_lower_case(input_object_name)} "
                    f"import {input_object_name}"
                )
            buffer.write("")

        sorted_objects = sorted(
            parsed_query.objects,
            key=lambda obj: 1 if isinstance(obj, ParsedOperation) else 0,
        )
        for obj in sorted_objects:
            buffer.write("")
            if isinstance(obj, ParsedObject):
                self.__render_object(parsed_query, buffer, obj)
            elif isinstance(obj, ParsedOperation):
                self.__render_operation(parsed_query, buffer, obj)

        if parsed_query.fragment_objects:
            if parsed_query.used_fragments:
                queries = [
                    f"{fragment_name}Query"
                    for fragment_name in sorted(set(parsed_query.used_fragments))
                ]
                buffer.write(f'QUERY: List[str] = {" + ".join(queries)} + ["""')
            else:
                buffer.write('QUERY: List[str] = ["""')
            buffer.write(parsed_query.query)
            buffer.write('"""]')
            buffer.write("")

        for fragment_obj in parsed_query.fragment_objects:
            self.__render_fragment(parsed_query, buffer, fragment_obj)

        return str(buffer)

    def render_enums(self, parsed_query: ParsedQuery) -> Dict[str, str]:
        result = {}

        for enum in parsed_query.enums + parsed_query.internal_enums:
            buffer = CodeChunk()
            buffer.write("#!/usr/bin/env python3")
            buffer.write("# @" + "generated AUTOGENERATED file. Do not Change!")
            buffer.write("")
            buffer.write("from enum import Enum")
            buffer.write("")
            with buffer.write_block(f"class {enum.name}(Enum):"):
                for value_name, value in enum.values.items():
                    if isinstance(value, str):
                        value = f'"{value}"'

                    buffer.write(f"{value_name} = {value}")
                buffer.write('MISSING_ENUM = ""')
                buffer.write("")
                buffer.write("@classmethod")
                with buffer.write_block(
                    f'def _missing_(cls, value: object) -> "{enum.name}":'
                ):
                    buffer.write("return cls.MISSING_ENUM")
            buffer.write("")
            result[enum.name] = str(buffer)

        return result

    def render_input_objects(self, parsed_query: ParsedQuery) -> Dict[str, str]:
        result = {}

        for input_object in parsed_query.input_objects + parsed_query.internal_inputs:
            buffer = CodeChunk()
            buffer.write("#!/usr/bin/env python3")
            buffer.write("# @" + "generated AUTOGENERATED file. Do not Change!")
            buffer.write("")
            buffer.write("from dataclasses import dataclass")
            buffer.write("from datetime import datetime")
            buffer.write("from functools import partial")
            buffer.write("from gql_client.runtime.datetime_utils import DATETIME_FIELD")
            buffer.write("from numbers import Number")
            buffer.write("from typing import Any, Callable, List, Mapping, Optional")
            buffer.write("")
            buffer.write("from dataclasses_json import DataClassJsonMixin")
            buffer.write("")
            enum_names = set()
            for enum in input_object.input_enums:
                enum_names.add(enum.name)
            if enum_names:
                buffer.write("from gql_client.runtime.enum_utils import enum_field")
                for enum_name in sorted(enum_names):
                    buffer.write(
                        f"from ..enum.{camel_case_to_lower_case(enum_name)} import {enum_name}"
                    )
                buffer.write("")
            input_object_names = set()
            for input_dep in input_object.inputs:
                input_object_names.add(input_dep.name)
            for input_object_name in sorted(input_object_names):
                buffer.write(
                    f"from ..input.{camel_case_to_lower_case(input_object_name)} "
                    f"import {input_object_name}"
                )

            self.__render_object(parsed_query, buffer, input_object, True)
            buffer.write("")
            result[input_object.name] = str(buffer)

        return result

    def __render_object(
        self,
        parsed_query: ParsedQuery,
        buffer: CodeChunk,
        obj: ParsedObject,
        is_input: bool = False,
    ) -> None:
        class_parents = (
            "(DataClassJsonMixin)" if not obj.parents else f'({", ".join(obj.parents)})'
        )

        buffer.write("@dataclass")
        with buffer.write_block(f"class {obj.name}{class_parents}:"):
            # render child objects
            children_names = set()
            for child_object in obj.children:
                if child_object.name not in children_names:
                    self.__render_object(parsed_query, buffer, child_object, is_input)
                children_names.add(child_object.name)

            # render fields
            sorted_fields = self.__sort_fields(parsed_query, obj.fields, is_input)
            for field in sorted_fields:
                self.__render_field(parsed_query, buffer, field, is_input)

            # pass if not children or fields
            if not (obj.children or obj.fields):
                buffer.write("pass")

        buffer.write("")

    def __render_fragment(
        self, parsed_query: ParsedQuery, buffer: CodeChunk, obj: ParsedObject
    ) -> None:
        class_parents = (
            "(DataClassJsonMixin)" if not obj.parents else f'({", ".join(obj.parents)})'
        )

        buffer.write("@dataclass")
        with buffer.write_block(f"class {obj.name}{class_parents}:"):

            # render child objects
            children_names = set()
            for child_object in obj.children:
                if child_object.name not in children_names:
                    self.__render_object(parsed_query, buffer, child_object)
                children_names.add(child_object.name)

            # render fields
            sorted_fields = self.__sort_fields(parsed_query, obj.fields)
            for field in sorted_fields:
                self.__render_field(parsed_query, buffer, field)

        buffer.write("")

    def __render_operation(
        self, parsed_query: ParsedQuery, buffer: CodeChunk, parsed_op: ParsedOperation
    ) -> None:
        if len(parsed_query.used_fragments):
            queries = [
                f"{fragment_name}Query"
                for fragment_name in sorted(set(parsed_query.used_fragments))
            ]
            buffer.write(f'QUERY: List[str] = {" + ".join(queries)} + ["""')
        else:
            buffer.write('QUERY: List[str] = ["""')
        buffer.write(parsed_query.query)
        buffer.write('"""]')
        buffer.write("")
        buffer.write("@dataclass")
        with buffer.write_block(f"class {parsed_op.name}(DataClassJsonMixin):"):
            # Render children
            for child_object in parsed_op.children:
                self.__render_object(parsed_query, buffer, child_object)

            # operation fields
            buffer.write(f"data: {parsed_op.name}Data")
            buffer.write("")

            # Execution functions
            if parsed_op.variables:
                vars_args = ", " + ", ".join(
                    [
                        self.__render_variable_definition(var)
                        for var in parsed_op.variables
                    ]
                )
                variables_dict = (
                    "{"
                    + ", ".join(
                        f'"{var.name}": {var.name}' for var in parsed_op.variables
                    )
                    + "}"
                )
            else:
                vars_args = ""
                variables_dict = "{}"

            buffer.write("@classmethod")
            buffer.write("# fmt: off")
            assert len(parsed_op.children) == 1
            child = parsed_op.children[0]
            assert len(child.fields) == 1
            query = child.fields[0]
            query_name = query.name
            query_result_type = f"{query.type}"
            if query_result_type not in ("int", "str", "bool"):
                query_result_type = f"{parsed_op.name}Data.{query.type}"
            if query.nullable:
                query_result_type = f"Optional[{query_result_type}]"
            if query.is_list:
                query_result_type = f"List[{query_result_type}]"
            with buffer.write_block(
                f"def execute(cls, client: GraphqlClient{vars_args})"
                f" -> {query_result_type}:"
            ):
                buffer.write("# fmt: off")
                buffer.write(f"variables: Dict[str, Any] = {variables_dict}")
                with buffer.write_block("try:"):
                    buffer.write("network_start = perf_counter()")
                    buffer.write(
                        "response_text = client.call(''.join(set(QUERY)), "
                        "variables=variables)"
                    )
                    buffer.write("decode_start = perf_counter()")
                    buffer.write("res = cls.from_json(response_text).data")
                    buffer.write("decode_time = perf_counter() - decode_start")
                    buffer.write("network_time = decode_start - network_start")
                    buffer.write(
                        f'client.reporter.log_successful_operation("{parsed_op.name}", variables, network_time, decode_time)'
                    )
                    buffer.write(f"return res.{query_name}")
                with buffer.write_block("except OperationException as e:"):
                    with buffer.write_block("raise FailedOperationException("):
                        buffer.write("client.reporter,")
                        buffer.write("e.err_msg,")
                        buffer.write(f'"{parsed_op.name}",')
                        buffer.write("variables,")
                    buffer.write(")")
            buffer.write("")

    @staticmethod
    def __sort_fields(
        parsed_query: ParsedQuery, fields: List[ParsedField], is_input: bool = False
    ) -> List[ParsedField]:
        enum_names = [e.name for e in parsed_query.enums + parsed_query.internal_enums]

        def sort_key(field) -> int:
            if field.type in enum_names or field.type == "DateTime":
                return 1
            if field.nullable and is_input:
                return 2
            return 0

        return sorted(fields, key=sort_key)

    @staticmethod
    def __render_variable_definition(var: ParsedVariableDefinition):
        var_type = var.type

        if var_type == "DateTime":
            var_type = "datetime"
        elif var_type == "Cursor":
            var_type = "str"

        if var.is_list:
            return f"{var.name}: List[{var_type}] = []"

        if not var.nullable:
            return f"{var.name}: {var_type}"

        return f'{var.name}: Optional[{var_type}] = {var.default_value or "None"}'

    @staticmethod
    def __render_field(
        parsed_query: ParsedQuery,
        buffer: CodeChunk,
        field: ParsedField,
        is_input: bool = False,
    ) -> None:
        enum_names = [e.name for e in parsed_query.enums + parsed_query.internal_enums]
        is_enum = field.type in enum_names
        suffix = ""
        field_type = field.type
        if field.is_list:
            field_type = f"List[{field_type}]"

        if is_enum:
            suffix = f" = enum_field({field_type})"

        if field_type == "DateTime":
            suffix = " = DATETIME_FIELD"
            field_type = "datetime"

        if field.nullable:
            if is_input:
                suffix = f" = {field.default_value}"
            buffer.write(f"{field.name}: Optional[{field_type}]{suffix}")
        else:
            buffer.write(f"{field.name}: {field_type}{suffix}")



#!/usr/bin/env python3

import warnings
from logging import Logger, getLogger
from typing import Any, Dict, Optional, cast

from graphql import (
    build_ast_schema,
    build_client_schema,
    get_introspection_query,
    parse,
)
from graphql.language.ast import DocumentNode
from graphql.type.schema import GraphQLSchema
from graphql.utilities.find_deprecated_usages import find_deprecated_usages
from graphql.validation import validate

from .transport.local_schema import LocalSchemaTransport
from .transport.transport import ExtendedExecutionResult, Transport


log: Logger = getLogger(__name__)


class OperationException(Exception):
    def __init__(self, err_msg: str) -> None:
        super(OperationException, self).__init__(f"Operation failed: {err_msg}")
        self.err_msg = err_msg


class RetryError(Exception):
    """Custom exception thrown when retry logic fails"""

    def __init__(self, retries_count: int, last_exception: Optional[Exception]) -> None:
        message = "Failed %s retries: %s" % (retries_count, last_exception)
        super(RetryError, self).__init__(message)
        self.last_exception = last_exception


class GraphqlDeprecationWarning(DeprecationWarning):
    pass


class Client(object):
    schema: Optional[GraphQLSchema]
    introspection: Optional[Dict[str, Any]]
    transport: Transport
    retries: int

    def __init__(
        self,
        schema: Optional[GraphQLSchema] = None,
        introspection: Optional[Dict[str, Any]] = None,
        type_def: Optional[str] = None,
        transport: Optional[Transport] = None,
        fetch_schema_from_transport: bool = False,
        retries: int = 0,
    ) -> None:
        assert not (
            type_def and introspection
        ), "Cant provide introspection type definition at the same time"
        if transport and fetch_schema_from_transport:
            assert (
                not schema
            ), "Cant fetch the schema from transport if is already provided"
            introspection = transport.execute(
                parse(get_introspection_query(descriptions=True))
            ).data
        if introspection:
            assert not schema, "Cant provide introspection and schema at the same time"
            schema = build_client_schema(introspection)
        elif type_def:
            assert (
                not schema
            ), "Cant provide Type definition and schema at the same time"
            type_def_ast = parse(type_def)
            schema = build_ast_schema(type_def_ast)
        elif schema and not transport:
            transport = LocalSchemaTransport(schema)

        self.schema = schema
        self.introspection = introspection
        self.transport = cast(Transport, transport)
        self.retries = retries

    def validate(self, document: DocumentNode) -> None:
        schema = self.schema
        if not schema:
            raise Exception(
                "Cannot validate locally the document, you need to pass a schema."
            )
        validation_errors = validate(schema, document)
        if validation_errors:
            raise validation_errors[0]
        usages = find_deprecated_usages(schema, document)
        for usage in usages:
            message = (
                f"Query of deprecated grapqhl field in {usage}"
                "Consider upgrading to newer API version."
            )
            warnings.warn(message, GraphqlDeprecationWarning)

    def execute(self, document: DocumentNode, variable_values: Dict[str, Any]) -> str:
        if self.schema:
            self.validate(document)

        result = self._get_result(document, variable_values)
        if result.errors:
            raise OperationException(
                str(cast(Dict[int, str], result.errors)[0]),
            )

        return result.response

    def _get_result(
        self, document: DocumentNode, variable_values: Dict[str, Any]
    ) -> ExtendedExecutionResult:
        if not self.retries:
            return self.transport.execute(document, variable_values)

        last_exception = None
        retries_count = 0
        while retries_count < self.retries:
            try:
                result = self.transport.execute(document, variable_values)
                return result
            except Exception as e:
                last_exception = e
                log.warn(
                    "Request failed with exception %s. Retrying for the %s time...",
                    e,
                    retries_count + 1,
                    exc_info=True,
                )
            finally:
                retries_count += 1

        raise RetryError(retries_count, last_exception)



#!/usr/bin/env python3

from typing import Any, Dict, cast

from graphql.execution import ExecutionResult, execute
from graphql.language.ast import DocumentNode
from graphql.type.schema import GraphQLSchema

from .transport import ExtendedExecutionResult, Transport


class LocalSchemaTransport(Transport):
    def __init__(self, schema: GraphQLSchema) -> None:
        self.schema = schema

    def execute(
        self, document: DocumentNode, variable_values: Dict[str, Any] = {}  # noqa: B006
    ) -> ExtendedExecutionResult:
        result = cast(ExecutionResult, execute(self.schema, document))
        return ExtendedExecutionResult(
            response="", data=result.data, errors={}, extensions={}
        )



#!/usr/bin/env python3

from typing import Any, Dict, Optional

import requests
from requests.auth import AuthBase

from . import gql
from .client import Client
from .reporter import DUMMY_REPORTER, Reporter
from .transport.session import RequestsHTTPSessionTransport


class GraphqlClient:
    def __init__(
        self,
        graphql_endpoint_address: str,
        session: requests.Session,
        auth: Optional[AuthBase] = None,
        headers: Dict[str, str] = {},
        reporter: Reporter = DUMMY_REPORTER,
    ) -> None:

        """This is the class to use for working with graphql server

        Args:
            graphql_endpoint_address (str): The graphql server address
            auth (Optional[requests.auth.AuthBase], optional): Auth used
                to authenticate to graphql server
            reporter (object, optional): Use reporter.InventoryReporter to
                       store reports on all successful and failed mutations
                       in inventory. The default is DummyReporter that
                       discards reports

        """

        self.reporter = reporter
        self.client = Client(
            transport=RequestsHTTPSessionTransport(
                session,
                graphql_endpoint_address,
                headers={
                    "Accept": "application/json",
                    "Content-Type": "application/json",
                    **headers,
                },
                auth=auth,
            ),
            fetch_schema_from_transport=True,
        )

    def call(self, query: str, variables: Dict[str, Any]) -> str:
        return self.client.execute(gql(query), variable_values=variables)
